package com.cedarsoft.business.contact;

import com.cedarsoft.business.contact.communication.CommunicationChannel;
import com.cedarsoft.business.contact.communication.Email;
import com.cedarsoft.business.contact.communication.FaxNumber;
import com.cedarsoft.business.contact.communication.PhoneNumber;
import com.cedarsoft.history.ClusteredElementsCollection;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

/**
 *
 */
public class CommunicationChannels extends ClusteredElementsCollection<CommunicationChannel> {
  @NotNull
  @NonNls
  public static final String PROPERTY_FIRST_ACTIVE_FAX = "firstActiveFax";
  @NotNull
  @NonNls
  public static final String PROPERTY_FIRST_ACTIVE_MAIL = "firstActiveMail";
  @NotNull
  @NonNls
  public static final String PROPERTY_FIRST_ACTIVE_PHONE_NUMBER = "getFirstActivePhoneNumber";

  @NotNull
  private static final Log log = LogFactory.getLog( CommunicationChannels.class );

  @NotNull
  private static final Map<Class<? extends CommunicationChannel>, Integer> sorting = new HashMap<Class<? extends CommunicationChannel>, Integer>();

  static {
    sorting.put( Address.class, 5 );
    sorting.put( POBAddress.class, 6 );
    sorting.put( PhoneNumber.class, 10 );
    sorting.put( FaxNumber.class, 20 );
    sorting.put( Email.class, 30 );
  }

  @Override
  public void addElement( @NotNull CommunicationChannel element ) {
    lock.writeLock().lock();
    int index;
    try {
      elements.add( element );
      Collections.sort( elements, new Comparator<CommunicationChannel>() {
        public int compare( CommunicationChannel o1, CommunicationChannel o2 ) {
          return sorting.get( o1.getClass() ).compareTo( sorting.get( o2.getClass() ) );
        }
      } );

      index = elements.indexOf( element );
    } finally {
      lock.writeLock().unlock();
    }
    collectionSupport.elementAdded( element, index );
  }


  /**
   * Returns the first active fax (or creates a new one if necessary)
   *
   * @return the first active fax
   *
   * @throws IllegalStateException
   */
  @NotNull
  public FaxNumber getFirstActiveFax() throws IllegalStateException {
    try {
      return findFirstActiveChannel( FaxNumber.class );
    } catch ( IllegalStateException ignore ) {
    }

    FaxNumber fax = new FaxNumber( "??", "?????", "?????" );
    fax.setActive( true );
    addElement( fax );

    return fax;
  }

  @NotNull
  public Email getFirstActiveMail() {
    try {
      return findFirstActiveChannel( Email.class );
    } catch ( IllegalStateException ignore ) {
    }

    Email newChannel = new Email( "?????@????.com" );
    newChannel.setActive( true );
    addElement( newChannel );

    return newChannel;
  }

  @NotNull
  public PhoneNumber getFirstActivePhoneNumber() {
    try {
      return findFirstActiveChannel( PhoneNumber.class );
    } catch ( IllegalStateException ignore ) {
    }

    PhoneNumber newChannel = new PhoneNumber( "??", "?????", "?????" );
    newChannel.setActive( true );
    addElement( newChannel );

    return newChannel;
  }

  /**
   * Returns the first active fax
   *
   * @return the first active fax
   *
   * @throws IllegalStateException if no fax could be found
   */
  @NotNull
  public <T extends CommunicationChannel> T findFirstActiveChannel( @NotNull Class<T> type ) throws IllegalStateException {
    lock.readLock().lock();
    try {
      for ( CommunicationChannel entry : elements ) {
        if ( !entry.isActive() ) {
          continue;
        }
        if ( type.isAssignableFrom( entry.getClass() ) ) {
          return type.cast( entry );
        }
      }
    } finally {
      lock.readLock().unlock();
    }
    throw new IllegalStateException( "No active communication channel found with type " + type.getName() );
  }
}
